/**----------------------------------------------------------------------**/
/** Programm: bc_macro.sas                                               **/
/**    Project    :  sas-twophase-package                                **/
/**    Author     :  Schill, Seiler, Enders                              **/
/**    Date       :  1999-06-06                                          **/
/**    Update     :  2011-11-02  Macroparms DOKU, DEBUG deleted          **/
/**    Update     :  2012-01-04  Use of proc logistic instead of iml-    **/
/**                              routines for parameter estimation       **/
/**                                                                      **/
/**    Context    : sas-twophase-package                                 **/
/**    Purpose    : Parameter estimation                                 **/
/**    Method     : Pseudo Likelihood (Breslow/Cain)                     **/
/**    Procs      : proc logistic, IML                                   **/
/**    Include    : -                                                    **/
/**    Input      : &dat_ein (prepared input dataset)                    **/
/**    Output     : &dat_aus (result dataset)                            **/
/**                                                                      **/
/**    Validation : Enders                                               **/
/**    Status     :                                                      **/
/**    Doku       : Documentation                                        **/
/**    Reference  : Schill/Drescher (1997). Stats in Medicine            **/
/**                 Breslow/Cain (1988), Biometrika                      **/
/**----------------------------------------------------------------------**/

%MACRO BC_MACRO(
                DAT_EIN =prep,
                DAT_AUS =pl_bc_est,
                MAXITER =10,
                EPSILON =1E-10,
                PROSPS1 =0,
                REGRESS =altstr2 altstr3 altstr4);

 ***************************************************************************;
 * DAT_EIN : Name of prepared input dataset;
 * DAT_AUS : Name of result dataset;
 ***************************************************************************;
 * MAXITER : Max number of iterations                       ;
 * EPSILON : Accuracy of parameter estimation;
 ***************************************************************************;
 * PROSPS1 : Sample1 prospective? (0=no, 1=yes);
 * REGRESS : Regression variables;
 ***************************************************************************;

 %LET PRG_NAME=BC_MACRO.SAS;
 %LET DATETIME_START=%sysfunc(TIME());
 %put &DATETIME_START;
 %GLOBAL N_STRAT;

 %MACRO BC_LAUF;

   %BC_PREPS;
   %BC_PCLS;
   run;
 %MEND BC_LAUF;


 %MACRO BC_PREPS;

   /* read prepared data, build offset variable (prospective and retrospective sampling,
      Intercept and sums for event/trial-syntax*/

   %LET N_STRAT=.;
   data bc_temp1;set &DAT_EIN end=eof;
        if _SAMPLE=1 then do;
           _offset=(1-&PROSPS1)*log(_N1/_N0);
           _anz =_NZ0 + _NZ1;
           _alpha=0;
        end;
        if _SAMPLE=2 then do;
           _offset=log(_M1/_M0);
           _alpha=1;
           _anz =_MZ0 + _MZ1;
        end;
        if eof then call symput("N_STRAT",compress(left(put(_S,8.0))));
   run;

   %MACRO MODEL1;
      %DO i=1 %TO &N_STRAT; _S&i %END;
   %MEND;

   %MACRO MODEL2;
      _alpha  &REGRESS
   %MEND;


   %* Phase 1 data regression;
   proc logistic noprint data=bc_temp1(where=(_SAMPLE=1))
        outest=bc_temp2 (keep=%model1);
        model _NZ1/_anz=%model1 / noint offset=_offset
        ridging=absolute maxiter=&MAXITER absfconv=&EPSILON;
   run;

   %* Build new offsets for phase 2 regression;
   data _null_;set bc_temp2;
        %do i=1 %to &n_strat;
            call symput("par_&i",_S&i);
        %end;
   run;

   data bc_temp1;set bc_temp1;
      if _SAMPLE=2 then do;
        %do i=1 %to &n_strat;
            _offset=_offset - &&par_&i*_S&i;
        %end;
      end;
   run;

   %* logistic regression phase 2;
   proc logistic noprint data=bc_temp1(where=(_SAMPLE=2))
        outest=bc_temp3 (keep=%model2);
        model _MZ1/_anz  =%model2 / noint offset=_offset
              ridging=absolute maxiter=&MAXITER absfconv=&EPSILON;
   run;

   %* Combine estimates into one observation;
   data bc_temp2;
        set bc_temp2;
        set bc_temp3;
   run;
 %MEND BC_PREPS;


 %* Compute variance of Breslow/Cain-estimate;
 %MACRO BC_PCLS;
   proc iml;
    start input;
      use bc_temp1;
        read all var {_NZ0}    into wgt10  where(_SAMPLE=1);
        read all var {_NZ1}    into wgt11  where(_SAMPLE=1);
        read all var {_MZ0}    into wgt20  where(_SAMPLE=2);
        read all var {_MZ1}    into wgt21  where(_SAMPLE=2);
        read all var {%MODEL1} into z1     where(_SAMPLE=1);
        read all var {%MODEL1} into z2     where(_SAMPLE=2);
        read all var {%MODEL2} into x2     where(_SAMPLE=2);
        read all var {_offset} into offs1  where(_SAMPLE=1);
        read all var {_offset} into offs2  where(_SAMPLE=2);
      close bc_temp1;

      *dimensions;
      nz  =nrow(wgt10);
      nx  =nrow(wgt20);
      *responses case//control, counts and regressors;
      y1     = repeat(1,nz,1)//repeat(0,nz,1);
      y2     = repeat(1,nx,1)//repeat(0,nx,1);
      wgt1   = wgt11//wgt10;
      wgt2   = wgt21//wgt20;
      offset1= offs1//offs1;
      offset2= offs2//offs2;
      * total sample size;
      n      = sum(wgt11+wgt10);
      * numbers in Sample2 per stratum;
      n2     = t(z2);
      m20    = n2*wgt20;
      m21    = n2*wgt21;
      * Design matrices;
      z1     = z1//z1;
      z2     = z2//z2;
      x2     = x2//x2;
      *parm  - vector with parameter-names;
      parm = {%MODEL2};

      %* read estimates;
      use bc_temp2;
          read all into b;
      close bc_temp2;
      b=t(b);
      b1=b[1:&n_strat];
      b2=b[(&n_strat+1):nrow(b)];

      eta1=offset1+z1*b1;
      eta2=offset2+x2*b2;
      p1=1/(1+exp(-eta1));
      p2=1/(1+exp(-eta2));
      f1=p1#(1-p1);
      f2=p2#(1-p2);
      xpxi1 = inv(z1`*(wgt1#f1#z1));
    finish input;

    start varpclbc;

      %* Dimensions;
      j  = ncol(z1);    * =&n_strat;
      p  = ncol(x2);

      %* 1st stage data;
      E  = J(j,j,1);
      rho= n/sum(wgt10) + n/sum(wgt11);
      V  = diag(t(z1)*(f1#wgt1))/n;
      W  = inv(V) - (1-&prosps1)*rho*E;
      Cov1=W/n;
      stderr1= sqrt(vecdiag(COV1));

      %* 1st and 2nd stage data;
      x12= J(nrow(z1),p,0);
      X  = (z1||x12)//(-z2||x2);
      WGT= wgt11//wgt10//wgt21//wgt20;
      F  = f1//f2;
      H  = (X`*(WGT#F#X))/n;
      * Modification of variance due to 2-stage setup (BC);
      Q  = n*(inv(diag(m20)) + inv(diag(m21)));
      H12= H[1:j,j+1:j+p];
      H21= t(H12);
      H22= H[j+1:j+p,j+1:j+p];
      H22i= inv(H22);

      COV = H22i*(H22 - H21*(Q-W)*H12)*H22i/n;
      stderr = sqrt(vecdiag(COV));

    finish varpclbc;

    run input;
    run varpclbc;

    * Output rsults to files;
    COV   = block(COV,xpxi1);
    parm  = {%MODEL2 %MODEL1};
    b     = b2//b1;
    stderr= stderr//stderr1;
    create bc_temp1 var{parm b stderr};
      append;
    close bc_temp1;
    create bc_temp2 from COV[colname=parm];
      append from COV ;
    close bc_temp2;

    * Combine datasets;
    %LET FLAG=1;
    %* elapsed time since beginning of macro;
    %LET ELAPS=%sysfunc(putn(%sysevalf(%sysfunc(TIME())-&DATETIME_START.),E8601TM12.2)) (hh:mm:ss.ff) ;
    data  &DAT_AUS(label="Prgm: &PRG_NAME, &sysdate"
                             drop =_day _mon _year _hour _min _sec);
    merge bc_temp1(rename=(parm=_parms b=_estim stderr=_stderr))
          bc_temp2;
     if symget("FLAG") = 1 then do;
        call symput("FLAG","0");
        _program = "&PRG_NAME";
        _source  = "&path_ph1 (Phase 1), &path_ph2 (Phase 2)";
        _regress = "&REGRESS";
        _day     = compress(day(   date()));
        _mon     = compress(month( date()));
        _year    = compress(year(  date()));
        _hour    = compress(hour(  time()));
        _min     = compress(minute(time()));
        _sec     = compress(second(time()));
        if _day < 10 then _day =compress("0"||_day );
        if _mon < 10 then _mon =compress("0"||_mon );
        if _hour< 10 then _hour=compress("0"||_hour);
        if _min < 10 then _min =compress("0"||_min );
        if _sec < 10 then _sec =compress("0"||_sec );
        _time    = compress(_year || "." || _mon || "." || _day) || " " ||
                   compress(_hour || ":" || _min || ":" || _sec);
        _method  = "pl_bc";
        _elaps   = "&ELAPS";
      end;
      label
        _parms   = 'name of parameter'
        _estim   = 'estimate'
        _stderr  = 'standard error'
        _source  = 'source file used by _program'
        _regress = 'regressors used by _program'
        _program = 'program that created this file'
        _time    = 'time of creation of this file'
        _method  = 'method used by _program'
         _elaps  = 'time used by _program'
         ;
      ;
    run;
    proc datasets library=work nolist;
         delete bc_temp1 bc_temp2 bc_temp3;
    quit;

 %MEND BC_PCLS;

 %BC_LAUF;

%MEND BC_MACRO;
