/**--------------------------------------------------------------------**/
/** Programm: em5macro.sas                                             **/
/**    Project    : sas-twophase-package                               **/
/**    Author     : Schill, Drescher                                   **/
/**    Date       : 1995-07-15                                         **/
/**    Update     : 2011-11-02  Macroparms DOKU, DEBUG deleted         **/
/**    Update     : 2012-04-11  Speeding up Convergence (T Louis, 1982)**/
/**                 2012-07-24  Implementation retrospective phase one **/
/**                 2013-05-02  Warning if &maxit is exceeded          **/
/**                 2013-11-22  added output                           **/
/**                                                                    **/
/**    Context    : sas-twophase-package                               **/
/**    Purposea   : Parameter estimation                               **/
/**    Method     : ML via EM-Algorithm                                **/
/**    Procs      : IML                                                **/
/**    Include    : -                                                  **/
/**    Input      : &dat_ein (prepared input dataset)                  **/
/**    Output     : &dat_aus (result dataset)                          **/
/**                 em_design, em_final, em_theta                      **/
/**                                                                    **/
/**    Validation :                                                    **/
/**    Status     :                                                    **/
/**    Doku       : Documentation\TPMethods.pdf                        **/
/**    Reference  : Schill/Drescher (1997). Stats in Medicine          **/
/**--------------------------------------------------------------------**/

%MACRO EM5MACRO(
                DAT_EIN =prep,
                DAT_AUS =em_dsgn,
                MAXITER =1000,
                EPSILON =1E-10,
                PROSPS1 =0,
                REGRESS =altstr2 altstr3 altstr4);

 **********************************************************************;
 * DAT_EIN : Name of prepared input data;
 * DAT_AUS : Name of result dataset;
 * PROSPS1 : Sample 1 prospective ? (1=yes, 0=no)                      ;
 * MAXITER : Max number of iterations                                  ;
 * EPSILON : Accuracy of parameter estimation;
 **********************************************************************;
 * REGRESS : Regressor;
 **********************************************************************;

 %LET PRG_NAME=EM5MACRO.SAS;
 %LET DATETIME_START=%sysfunc(TIME());
 %GLOBAL VARS NP;

 %MACRO EM5_LAUF;
   %EM5PREPS;
   %EM5;
   run;
 %MEND EM5_LAUF;


 %MACRO EM5PREPS;

   %LET VARS=%STR(&REGRESS);
   %LET NP=0;
   %DO %WHILE(%SCAN(&VARS,&NP+1)^=);
     %LET NP=%EVAL(&NP+1);
   %END;

   data em_temp1;
        set &DAT_EIN(where=(_sample=2));
        by _S;
        _NM0 = _NZ0-_M0;       %* "Missings" under controls per stratum;
        _NM1 = _NZ1-_M1;       %* "Missings" under cases per stratum;
        _N   =_N_;             %* Rank order of sorted covariate pattern;
        if first._S then c=0;
        c+1;
        _parmc=compress('delta'||_S||'.'||c);

   run;

   data em_temp2;
      set em_temp1;
        keep y _n0 _n1 m_com m_inc _S &vars _N _parmc nz1;
        do Y=0 to 1;
           if Y=0 then do;
              m_com=_MZ0;
              m_inc=_NM0;
              nz1  =_NZ0;
              output;
           end;
           if Y=1 then do;
              m_com=_MZ1;
              m_inc=_NM1;
              nz1  =_NZ1;
              output;
           end;
        end;
   proc sort;
        by Y _S;
   run;

   data em_temp2;set em_temp2;
        _SS2=2*_S-1 + Y;   %* cell number in Phase 1 table;
   run;
 %MEND EM5PREPS;


 %MACRO EM5;
   proc iml ;
    start input;
      use em_temp2;;
          read all var {_N0}      into n0;
          read all var {_N1}      into n1;
          read all var {m_com}    into m_com ;
          read all var {m_inc}    into m_inc ;
          read all var {_S}       into z  ;
          read all var {_SS2}     into z2  ;
          read all var {_N}       into nr  ;
          read all var {&VARS}    into vars  ;
          read all var {Y}        into y ;
          read all var {_parmc}   into parmc where(Y=0);
          n0=n0[1];
          n1=n1[1];
      close em_temp2;

     %* parm  - vector with parameter-names
        theta - vector with parameter-values
        nxz   - number of covariate patterns;
      nxz  = nrow(y)/2;

      parma= {_alpha };
      parmb= { %DO I=1 %TO &NP;
                 %SCAN(&VARS,&I)
                 %IF &I < &NP %THEN ,;
               %END;
             };
      parm  =parma//parmb//parmc;
      thea  =log(n1/n0);
      theb  =repeat(0,&NP,1);
      thec  =repeat(log(n0/nxz),nxz,1);
      theta =thea//theb//thec;

      %* Design-matrix X for Poisson-model;
      X=y||(y=1)#vars||Design(nr);
    finish input;

     start em;
       run input;
       mu=exp(X*theta);

       %* To Computate weights (per z2 (Stratum x Y));
       B =Design(z2);
       BB=B*T(B);

       do iter=1 to &maxiter until (sum(abs(theta-thetaold))<&epsilon);
          %* Update weight vector w;
          w=mu/(BB*mu);
          %*Update pseudo-counts m;
          m=m_com + Diag(w)*m_inc;
          mlogm=repeat(0,nrow(X),1);
          do _i=1 to nrow(X);
             if m[_i]>0 then do;
                mlogm[_i]=m[_i]*log(m[_i]);
             end;
          end;
          devold=2*sum(mlogm - m#log(mu) - m + mu);
          thetaold=theta;

          if iter<10 then do;
             theta=theta + inv(T(X)*Diag(mu)*X)*T(X)*(m-mu);
          end;
          else do;
             WW =(w*T(w))#BB;
             COV=inv(T(X)*(diag(mu)-diag(m_inc)*(diag(w)-WW))*X);
             theta=theta + COV*T(X)*(m-mu);
          end;

          do _i=1 to 3 until (dev<devold + &epsilon);
             mu  =exp(X*theta);
             dev=2*sum(mlogm - m#log(mu) - m + mu);
             *print iter _i dev devold;
             if dev > devold + &epsilon then do;
                theta=(theta+thetaold)/2;
             end;
          end;

       end; %* end iter;
       WW =(w*T(w))#BB;
       COV=inv(T(X)*(diag(mu)-diag(m_inc)*(diag(w)-WW))*X);

       %* Modification of intercept due to retrospective phase one sample;
       if &prosps1=0 then do;
          theta[1]=theta[1] - log(n1/n0);
          COV[1,1]=COV[1,1] - (1/n0 + 1/n1);
       end;

       conv=1;
       if iter > &maxiter then do;
          conv=0;
          print '+++++++',
                "+ WARNING: EM algorithm did not meet convergence criterion (epsil= &epsilon)",
                "+ Probably more than maxit= &maxiter iterations are needed",
                '+++++++';
       end;

    finish em;
    run em;

    parm2=parma//parmb;
    b=theta[1:nrow(parm2)];
    COVb=COV[1:nrow(parm2),1:nrow(parm2)];
    stderr=sqrt(vecdiag(COV));
    stderrb=stderr[1:nrow(parm2)];

    * Output into data files;
    create em_temp1 var{parm2 b stderrb};
      append;
    close em_temp1;
    create em_temp2 from COVb[colname=parm2];
      append from COVb;
    close em_temp2;

    create em_temp3 var{ parm theta stderr};
      append;
    close em_temp3;
    create em_temp4 from COV[colname=parm];
      append from COV;
    close em_temp4;
    create em_temp5 var{z y m_com m_inc mu};
      append;
    close em_temp5;
    create em_temp6 from vars[colname=parmb];
      append from vars;
    close em_temp6;
    create em_temp7 var{conv};
      append;
    close em_temp7;
    create em_temp8 from X [colname=parm];
      append from X;
    close em_temp8;

    %LET FLAG=1;
    %LET ELAPS=%sysfunc(putn(%sysevalf(%sysfunc(TIME())-&DATETIME_START.),E8601TM12.2)) (hh:mm:ss.ff) ;


    data  &DAT_AUS(label="Prgm: &PRG_NAME, &sysdate"
          drop =_day _mon _year _hour _min _sec);
    merge em_temp1(rename=(parm2=_parms b=_estim stderrb=_stderr))
          em_temp2;
      if symget("FLAG") = 1 then do;
        call symput("FLAG","0");
        _program = "&PRG_NAME";
        _source  = "&path_ph1 (Phase 1), &path_ph2 (Phase 2)";
        _regress = "&REGRESS";
        _mon     = compress(month( date()));
        _year    = compress(year(  date()));
        _hour    = compress(hour(  time()));
        _min     = compress(minute(time()));
        _sec     = compress(second(time()));
        if _day < 10 then _day =compress("0"||_day );
        if _mon < 10 then _mon =compress("0"||_mon );
        if _hour< 10 then _hour=compress("0"||_hour);
        if _min < 10 then _min =compress("0"||_min );
        if _sec < 10 then _sec =compress("0"||_sec );
        _time    = compress(_year || "." || _mon || "." || _day) || " " ||
                   compress(_hour || ":" || _min || ":" || _sec);
        _method  = "ml_em";
        _elaps   = "&ELAPS";
      end;
      label
        _parms   = 'name of parameter'
        _estim   = 'estimate'
        _stderr  = 'standard error'
        _source  = 'source file used by _program'
        _regress = 'regressors used by _program'
        _program = 'program that created this file'
        _time    = 'time of creation of this file'
        _method  = 'method used by _program'
        _elaps   = 'time used by _program'
      ;
    run;

    data EM_Theta&suffix(label="Prgm: &PRG_NAME, &sysdate");
         merge em_temp3(rename=(parm=_parms Theta=_estim stderr=_stderr))
               em_temp4;
    run;
    data EM_Final&suffix(label="Prgm: &PRG_NAME, &sysdate");
         merge em_temp5 em_temp6;
         label z  ='Stratum'
               mu ='Expected Count'
         ;
    run;
    data EM_Design&suffix(label="Prgm: &PRG_NAME, &sysdate");
         set em_temp8;
         ;
    run;

    %* Issue a message if convergence criterion was not met;
    %let conv=1;
    data _null_;set em_temp7;
         if conv=0 then
         call symput('conv',conv);
    run;
    %if &conv ne 1 %then %do;
        %put   '+++++++';
        %put   "+ WARNING: EM algorithm did not meet convergence criterion (epsil= &epsilon)";
        %put   "+ Probably more than maxit= &maxiter iterations are needed";
        %put   '+++++++';
    %end;

    proc datasets library=work nolist;
         delete em_temp1 em_temp2 em_temp3 em_temp4 em_temp5 em_temp6 em_temp7 em_temp8;
   quit;

 %MEND EM5;

 %EM5_LAUF;

%MEND EM5MACRO;
