/*******************************************************************************
*                                                                              *
** PROGRAM NAME: prep.sas                                                      *
** AUTHOR: D. Enders                                                           *
** VERSION/DATE: V 1.0 / 31.10.2011                                            *
*                                                                              *
** PURPOSE: Data preparation in two-stage studies                              *
*                                                                              *
** PROGRAM SPECIFICATION: Subroutine in Macro twophase.sas                     *
*                         counts of cases/controls for each stratum in phase 2 *
*                         Append phase-one to phase 2 Data                     *
*                         assigns Missings to zero                             *
*                                                                              *
** -READ: &DAT_EIN_ph1 (specified in twophase.sas)                             *
*         &DAT_EIN_ph2 (specified in twophase.sas)                             *
** -WRITE: &DAT_AUS    (specified in twophase.sas)                             *
*                                                                              *
** MODIFICATION NOTES:                                                         *
*                V1.0 - V2.0: NA                                               *
** VALIDATION PROCESS:                                                         *
*                Second program review by W Schill                             *
*                                                                              *
** REFERENCE FOR DOCUMENTATION: NA                                             *
*                                                                              *
********************************************************************************/

  %macro prep(
            DAT_EIN_ph1 =,         /* phase1-data as input data */
            DAT_EIN_ph2 =,         /* phase2-data as input data */
            DAT_AUS     =prep,     /* prepared data for analysis */
            strat       =,         /* stratum-variable */
            count       =,         /* phase-one observation counts */
            caco        =,         /* case-control variable*/
            case        =1,        /* identification of diseased (case)    */
            control     =0,        /* identification nondiseased (control) */
            covar       =,         /* covariates in regression model */
            weight_var  =          /* phase-two observations weights */
                        );
   %macro check(datin=,datout=,phase=,count=);
   /*------------------------------------------------------------------
      Macro does rudimentary consistency checks on input datasets
            outcome variable takes on only prespecified values
            stratum vriable is integer and is > 0
            count variable is not negative
            regressor variables in phase 2 dataset are not missing
   ------------------------------------------------------------------*/

      %if &phase=2 %then %do;
         %let np=0;
         %do %while(%scan(&covar,&np+1)^=);
            %let np=%eval(&np+1);
         %END;
      %end;

      %let outf=0;%let stratf=0;%let countf=0;%let covarf=0;
      data &datout;set &datin;
        _za=0;_zb=0;_zc=0;_zd=0;
        if &caco not in (&control &case) then do;
           _za=1;
           _za_+1;
           call symput('outf',trim(left(_za_)));
        end;
        if &strat ne int(&strat) or &strat <=0 then do;
           _zb=1;
           _zb_+1;
           call symput('stratf',trim(left(_zb_)));
        end;
        %if %length(&count)>0 %then %do;
         if &count <= 0 then do;
           _zc=1;
           _zc_+1;
           call symput('countf',trim(left(_zc_)));
         end;
        %end;
        %if &phase=2 %then %do;
            %do i=1 %to &np;
               if %scan(&covar,&i)=. then do;
                  _zd=1;
                  _zd_+1;
                  call symput('covarf',trim(left(_zd_)));
               end;
            %end;
        %end;
      run;
      %if &outf>0 %then %do;
          %put '+++++++';
          %put "+ WARNING: &outf record(s) in dataset &datin with outcome value(s) (varible %upcase(&caco)) out of range";
          %put '+ The record(s) will be removed for further processing';
          %put '+++++++';
          run;
          proc iml;
           print '+++++++',
                 "+ WARNING: &outf record(s) in dataset &datin with outcome value(s) (variable %upcase(&caco)) out of range",
                 '+ The record(s) will be removed for further processing',
                 '+++++++';
          quit;
      %end;
      %if &stratf>0 %then %do;
          %put '+++++++';
          %put "+ WARNING: &stratf record(s) in dataset &datin with invalid stratum value(s) (variable %upcase(&strat))";
          %put '+ The record(s) will be removed for further processing';
          %put '+++++++';
          run;
          proc iml;
           print '+++++++',
                 "+ WARNING: &stratf record(s) in dataset &datin with invalid stratum value(s) (variable %upcase(&strat))",
                 '+ The record(s) will be removed for further processing',
                 '+++++++';
          quit;
      %end;
      %if &countf>0 %then %do;
          %put '+++++++';
          %put "+ WARNING: &countf record(s) in dataset &datin with invalid frequency value(s) (variable %upcase(&count))";
          %put '+ The record(s) will be removed for further processing';
          %put '+++++++';
          run;
          proc iml;
           print '+++++++',
                 "+ WARNING: &countf record(s) in dataset &datin with invalid frequency value(s) (variable %upcase(&count))",
                 '+ The record(s) will be removed for further processing',
                 '+++++++';
          quit;
      %end;
      %if &covarf>0 %then %do;
          %put '+++++++';
          %put "+ WARNING: &covarf missing covariate value(s) in dataset &datin detected.";
          %put '+ The respective record(s) will be removed for further processing';
          %put '+++++++';
          run;
          proc iml;
           print '+++++++',
                 "+ WARNING: &covarf missing covariate value(s) in dataset &datin detected.",
                 '+ The respective record(s) will be removed for further processing',
                 '+++++++';
          quit;
      %end;
      data &datout;set &datout;
           drop _za _zb _zc _zd
           %if &outf>0   %then _za_
           %if &stratf>0 %then _zb_
           %if &countf>0 %then _zc_;
           %if &covarf>0 %then _zd_;;
           if _za or _zb or _zc or _zd then delete;
      run;

      %if &outf>0 or &stratf>0 or &countf>0 or &covarf>0 %then %do;
         *proc print data=&datout;
         title"Data &datin, cleaned";
         proc freq data=&datout;
              tables &caco*&strat/nopercent norow nocol;
              %if %length(&count) >0 %then
              weight &count;;
         run;
      %end;
   %mend check;
   %check(datin=&DAT_EIN_PH1,datout=prep_temp1,phase=1,count=&count);
   %check(datin=&DAT_EIN_PH2,datout=prep_temp2,phase=2,count=&weight_var);

   /*------------------------------------------------------
     Phase one preparation: Observation counts in Phase one
      _NZ1 - # Cases in each Stratum
      _NZ0 - # Controls in each Stratum
      _N1  - total number of cases
      _N0  - total number of controls
   ------------------------------------------------------*/

   %* Rearrange phase 1 data;
   data prep_temp1 (rename=(&count = _NZ1))
        prep_temp0 (rename=(&count = _NZ0));
        set prep_temp1;
        if &caco=&case then output prep_temp1;
        else                output prep_temp0;
   run;

   proc sort data=prep_temp0; by &strat; run;
   proc sort data=prep_temp1; by &strat; run;

   data prep_temp5;
        retain sum0 0 sum1 0;
        merge prep_temp0 prep_temp1;
        by &strat;
        sum0=sum0+_NZ0;
        sum1=sum1+_NZ1;
        call symput('sum_control',sum0);
        call symput('sum_case',sum1);
        call symput('nstrat',trim(&strat));
   run;

   data prep_temp5;set prep_temp5;
        _N0=&sum_control;
        _N1=&sum_case;
        _NSTRAT=&nstrat;
        _SAMPLE=1;
        keep &strat _N0 _N1 _NZ0 _NZ1 _NSTRAT _SAMPLE;
   run;

   /*------------------------------------------------------
     Phase two preparation: Observation counts in Phase two
     _MZ1 - # Cases per Stratum and unique covar-pattern
     _MZ0 - # Controls per Stratum and unique covar-pattern
     _M1  - # Cases per Stratum
     _M0  - # Controls per Stratum
   ------------------------------------------------------*/

   %* assume weights=1 if weight-variable is missing;
   %if &weight_var eq %then %do;
        data prep_temp1;
          set prep_temp2;
          _weight=1;
        run;
   %end;
   %else %do;
        data prep_temp1 (rename=(&weight_var=_weight));
          set prep_temp2;
        run;
   %end;

   proc sort data=prep_temp1;
        by &caco &strat &covar;
   run;

   %* Frequency of observations, weighted;
   proc means data=prep_temp1 noprint;
           var _weight;
           by &caco &strat &covar;
           output out=prep_temp2(keep=&strat &caco &covar count)
                  sum=count;
   run;

   %* Stratum X Disease Frequencies;
   proc means data=prep_temp1 noprint;
        var _weight;
        by &caco &strat;
        output out=prep_temp3(keep=&strat &caco count) sum=count;
   run;

   data prep_temp1(rename=(count=_MZ1))
        prep_temp2(rename=(count=_MZ0));
        set prep_temp2;
        if &caco=&case then output prep_temp1;
        else                output prep_temp2;
   run;


   data prep_temp3(rename=(count=_M1))
        prep_temp4(rename=(count=_M0));
        set prep_temp3;
        if &caco=&case then output prep_temp3;
        else                output prep_temp4;
   run;

   data prep_temp1;
        merge prep_temp1 prep_temp2;
        by &strat &covar;
   run;

   data prep_temp3;
        merge prep_temp3 prep_temp4;
        by &strat;
   run;

   data prep_temp4;
        merge prep_temp1 prep_temp3;
        by &strat;
   run;

   data prep_temp1;
        merge prep_temp4 prep_temp5;
        by &strat;
        _SAMPLE=2;
   run;

   %* append Phase-one-Data;
   data prep_temp1;
        set prep_temp5 prep_temp1;
   run;

   %* construct dummy-variabes for &strat;
   proc sort data=prep_temp1;
        by &strat;
   run;

   data prep_temp1;
        set prep_temp1;
        drop &caco;
        _S=&strat;
        %do i=1 %to &nstrat;
                _S&i=0;
            if &strat=&i then _S&i=1;
        %end;
   run;

   proc sort data=prep_temp1;
           by _sample _S &covar;
   run;

   %*generate Output Data, replace Missing by Zero;
   data &DAT_AUS;set prep_temp1;
     array change _numeric_;
     do over change;
        if change=. then change=0;
     end;
     label
      _NZ1    = '# Cases per Phase-1-Stratum'
      _NZ0    = '# Controls  per Phase-1-Stratum'
      _N1     = '# Cases in Phase 1'
      _N0     = '# Controls in Phase 1'
      _NSTRAT = '# Strata'
      _MZ1    = '# Cases per Stratum and unique covar-pattern in Phase 2'
      _MZ0    = '# Controls per Stratum and unique covar-pattern in Phase 2'
      _M1     = '# Cases per Stratum in Phase 2'
      _M0     = '# Controls per Stratum in Phase 2'
      _S      = 'Stratum-variable'
      %do i=1 %to &nstrat;
      _S&i    = "Indicator Stratum &i"
      %end;
      _SAMPLE = 'Phase (1/2: Phase 1/Phase 2)'
           ;

   run;
   proc datasets library=work nolist;
        delete prep_temp0 prep_temp1 prep_temp2 prep_temp3 prep_temp4 prep_temp5;
   quit;
%mend prep;
