/**---------------------------------------------------------------------**/
/** Programm: S2_MACRO.SAS                                              **/
/**    Project    : sas-twophase-package                                **/
/**    Author     : Seiler, Enders                                      **/
/**    Date       : 1999-06-10                                          **/
/**    Update     : 2011-11-02  Macroparms DOKU, DEBUG deleted          **/
/**    Update     : 2012-08-01  Replace IML-estimation routine with     **/
/**                             proc logistic                           **/
/**                                                                     **/
/**    Context    : sas-twophase-package                                **/
/**    Purpose    : Complete Case Analysis                              **/
/**    Method     : Maximum Likkelihood                                 **/
/**    Procs      : proc logistic, IML                                  **/
/**    Include    : -                                                   **/
/**    Input      : &dat_ein (prepared input dataset)                   **/
/**    Output     : &dat_aus (result dataset)                           **/
/**                                                                     **/
/**    Validation : Enders                                              **/
/**    Status     :                                                     **/
/**    Doku       : Dokumentation                                       **/
/**    Reference  : Schill/Drescher (1997). Stats in Medicine           **/
/**---------------------------------------------------------------------**/

%MACRO S2_MACRO(
                DAT_EIN =prep,
                DAT_AUS =s2_dsgn,
                MAXITER =100,
                EPSILON =1E-10,
                REGRESS =altstr2 altstr3 altstr4);

 *************************************************************************;
 * DAT_EIN : Name of prepared input data;
 * DAT_AUS : Name of result dataset;
 *************************************************************************;
 * MAXITER : max number of iterations;
 * EPSILON : Accuracy of parameter estimation;
 *************************************************************************;
 * REGRESS : Regression variables;
 *************************************************************************;

 %LET PRG_NAME=S2_MACRO.SAS;
 %LET DATETIME_START=%sysfunc(TIME());
 %GLOBAL N_STRAT VARS;

 %MACRO S2_LAUF;
   %S2_PREPS;
   %S2_VAR;
 %MEND S2_LAUF;

 %MACRO S2_PREPS;

   %MACRO MODEL;
          _alpha
          &REGRESS
   %MEND;

   %* Determine number of strata;
   %LET N_STRAT=.;
   data s2_temp1;
    set &DAT_EIN(where=(_SAMPLE=2));
     _alpha=1;
     * Sample2 stets retrospektiv
        => Varianz fr _alpha unten in IML berechnet...;
     _anz =_MZ0 + _MZ1;
     call symput("N_STRAT",compress(left(put(_S,8.0))));
   run;

   %* Compute phase 2 stratum sizes;
   proc means data=s2_temp1 noprint sum;
        var _MZ0 _MZ1;
        output out=s2_temp2(keep=_sm20 _sm21) sum=_sm20 _sm21;
   run;

    data s2_temp1;
         if _N_=1 then set s2_temp2;
         set s2_temp1;
         _offset=log(_sm21/_sm20);
    run;

    proc logistic data=s2_temp1 noprint
         outest=s2_temp2(keep= %model);
         model _MZ1/_anz= %model / offset=_offset noint
               ridging=absolute maxiter=&maxiter absfconv=&epsilon;
    run;
 %MEND S2_PREPS;


 %MACRO S2_VAR;

   proc iml;
    start input;
      use s2_temp1;
        read all var {_MZ0}    into wgt20  ;
        read all var {_MZ1}    into wgt21  ;
        read all var {_sm20}   into sm20   ;
        read all var {_sm21}   into sm21   ;
        read all var {%MODEL}  into x      ;
        read all var {_OFFSET} into offset ;
      close s2_temp1;

      *dimensions;
      nx=nrow(wgt20);
      %*responses case//control, counts and regressors;
      y      = repeat(1,nx,1)//repeat(0,nx,1);
      wgt    = wgt21//wgt20;
      offset = offset//offset;
      %* Design matrix;
      x      = x//x;
      %* Phase 2 sample sizes;
      sum20=sm20[1];
      sum21=sm21[1];

      %* Read estimated parameters;
      use s2_temp2;
        read all into b ;
      close s2_temp2;
      b=t(b);

      *parm  - vector with parameter-names;
      parm = {%MODEL};
    finish input;

    start var_s2;

      eta = offset + x*b;
      p = 1/(1+exp(-eta));
      f = p#(1-p);
      xpxi = inv(x`*(wgt#f#x));

      * Sample2 stets retrospektiv => Korrektur von Var(_alpha);
      xpxi[1,1] = xpxi[1,1]  -  (1/sum20 + 1/sum21);
      cov=xpxi;
      stderr= sqrt(vecdiag(cov));
    finish var_s2;

    run input;
    run var_s2;

    * Output to dataset;
    create s2_temp1 var{parm b stderr};
      append;
    close s2_temp1;
    create s2_temp2 from COV[colname=parm];
      append from COV ;
    close s2_temp2;

    * Ausgabedateien zusammenfgen;
    %LET FLAG=1;
        %LET ELAPS=%sysfunc(putn(%sysevalf(%sysfunc(TIME())-&DATETIME_START.),E8601TM12.2)) (hh:mm:ss.ff) ;
    data  &DAT_AUS(label="Prgm: &PRG_NAME, &sysdate"
                             drop =_day _mon _year _hour _min _sec);
    merge s2_temp1(rename=(parm=_parms b=_estim stderr=_stderr))
          s2_temp2;
      if symget("FLAG") = 1 then do;
        call symput("FLAG","0");
        _program = "&PRG_NAME";
        _source  = "&path_ph1 (Phase 1), &path_ph2 (Phase 2)";
        _regress = "&REGRESS";
        _day     = compress(day(   date()));
        _mon     = compress(month( date()));
        _year    = compress(year(  date()));
        _hour    = compress(hour(  time()));
        _min     = compress(minute(time()));
        _sec     = compress(second(time()));
        if _day < 10 then _day =compress("0"||_day );
        if _mon < 10 then _mon =compress("0"||_mon );
        if _hour< 10 then _hour=compress("0"||_hour);
        if _min < 10 then _min =compress("0"||_min );
        if _sec < 10 then _sec =compress("0"||_sec );
        _time    = compress(_year || "." || _mon || "." || _day) || " " ||
                   compress(_hour || ":" || _min || ":" || _sec);
        _method  = "s2";
        _elaps   = "&ELAPS";
      end;
      label
        _parms   = 'name of parameter'
        _estim   = 'estimation'
        _stderr  = 'standard error'
        _source  = 'source file used by _program'
        _regress = 'regressors used by _program'
        _program = 'program that created this file'
        _time    = 'time of creation of this file'
        _method  = 'method used by _program'
        _elaps   = "&ELAPS";
      ;
    run;
    proc datasets library=work nolist;
         delete s2_temp1 s2_temp2;
    quit;

 %MEND S2_VAR;

 %S2_LAUF;

%MEND S2_MACRO;
