/**---------------------------------------------------------------------**/
/** Programm: wlrmacro.sas                                              **/
/**    Project    : sas-twophase-package                                **/
/**    Author     : Schill, Seiler                                      **/
/**    Date       : 1995-06-06                                          **/
/**    Update     : 2011-11-02  Macroparms DOKU, DEBUG deleted          **/
/**    Update     : 2012-07-23  Modification for retrospective phase 1  **/
/**    Update     : 2012-08-01  Replace IML-estimation routine with     **/
/**                             proc logistic                           **/
/**                                                                     **/
/**    Context    : sas-twophase-package                                **/
/**    Purpose    : Parameter estimation                                **/
/**    Method     : Weighted Likelihood (mean score method)             **/
/**    Procs      : proc logistic, IML                                  **/
/**    Include    : -                                                   **/
/**    Input      : &dat_ein (prepared input dataset)                   **/
/**    Output     : &dat_aus (result dataset)                           **/
/**                                                                     **/
/**    Validation : Enders                                              **/
/**    Status     :                                                     **/
/**    Doku       : Dokumentation                                       **/
/**    Reference  : Schill/Drescher (1997). Stats in Medicine           **/
/**                 Flanders/Greenland, Reilly/Pepe                     **/
/**---------------------------------------------------------------------**/

%MACRO WLRMACRO(
                DAT_EIN =prep,
                DAT_AUS =wl_est,
                MAXITER =20,
                EPSILON =1E-10,
                PROSPS1 =1,
                REGRESS =altstr2 altstr3 altstr4);

 *************************************************************************;
 * DAT_EIN : Name of prepared input data;
 * DAT_AUS : Name of results dataset;
 *************************************************************************;
 * MAXITER : Max number of iterations                       ;
 * EPSILON : Accuracy of parameter estimation;
 **PROSPS1 : Sample 1 prospective? (0=no, 1=yes)                          ;
 * REGRESS : Regressor;
 *************************************************************************;

 %LET PRG_NAME=WLRMACRO.SAS;
 %LET DATETIME_START=%sysfunc(TIME());
 %GLOBAL N_STRAT VARS NP;

 %MACRO WLR_LAUF;

   %WLRPREPS;
   %WLR;
   run;

 %MEND WLR_LAUF;


 %MACRO WLRPREPS;

   %LET VARS=%STR(&REGRESS);
   %LET NP=0;
   %DO %WHILE(%SCAN(&VARS,&NP+1)^=);
     %LET NP=%EVAL(&NP+1);
   %END;

   %* Compute #Missings per Stratum and Pseudo-complete counts per
      covariate pattern;

   %LET N_STRAT=.;
   data wl_temp1;
    set &DAT_EIN(where=(_SAMPLE=2)) end=eof;
     _NM0 = _NZ0-_M0;       %* "Missings" fr Kontrollen = #S1-#S2 per Stratum;
     _NM1 = _NZ1-_M1;       %* "Missings" fr Flle      = #S1-#S2 per Stratum;
     _w0  = _NZ0/_M0*_MZ0;  %* Pseudo-counts, controls, per covariate pattern;
     _w1  = _NZ1/_M1*_MZ1;  %* Pseudo-counts, cases, per covariate pattern;
     _anz =_w0+_w1;         %* Pseudo-count at risk, per covariate pattern;
     if eof then call symput("N_STRAT",compress(left(put(_S,8.0))));
   run;

   proc logistic data=wl_temp1 noprint
        outest=wl_temp2(keep=intercept &regress);
        model _w1/_anz= &regress/ ridging=absolute maxiter=&MAXITER absfconv=&EPSILON;
   run;
 %MEND WLRPREPS;


 %MACRO WLR;

   proc iml ;
    start input;
      use wl_temp1;
        %DO I=1 %TO &N_STRAT;
          read all var {_N1}    into n1     where(_S=&I);
          read all var {_N0}    into n0  where(_S=&I);
          read all var {_MZ1}   into m1_&I  where(_S=&I);
          read all var {_MZ0}   into m0_&I  where(_S=&I);
          read all var {_S}     into z_&I   where(_S=&I);
          read all var {&VARS}  into x_&I   where(_S=&I);
          read all var {_NM1}   into nm1_&I where(_S=&I);
          read all var {_NM0}   into nm0_&I where(_S=&I);
          *dimensions;
          nxz&I=nrow(m1_&I);
          n&I=2*nxz&I;
          *responses, counts and regressors;
          y_&I=repeat(1,nxz&I,1)//repeat(0,nxz&I,1);
          m_&I=m1_&I//m0_&I;
          nm_&I=nm1_&I//nm0_&I;
          z_&I=z_&I//z_&I;
          x_&I=x_&I//x_&I;
        %END;
        n0=n0[1];
        n1=n1[1];
      close wl_temp1;

      %* read estimates;
      use wl_temp2;
          read all into b;
      close wl_temp2;
      b=t(b);

      *parm  - vector with parameter-names
      theta - vector with parameter-values;
      parma= { _ALPHA };
      parmb= { %DO I=1 %TO &NP;
                 %SCAN(&VARS,&I)
                 %IF &I < &NP %THEN ,;
               %END;
             };
      parm=parma//parmb;
      *print parm y_1 y_2 m_1 m_2 nm_1 nm_2 z_1 z_2 x_1 x_2;
    finish input;

    start weight;
    /*  input: Sample  with variables
              y_&i  -  case-control-status
              m_&i  -  counts complete obs
              z_&i  -  stratum
              x_&i  -  regressors in stratum
              nm_&i -  counts incomplete obs

      output: X     - design-matrix
              y     - dependent
              m     - completed counts
              mcom  - complete obs
              mcomsd- complete obs in stratum X disease group
              mincsd- incompl. obs in stratum X disease group
     --------------------------------------------------------*/


      %do i=1 %to &n_strat;

      *Select stratum-specific observations;
        y_      = y_&i;
        m_com   = m_&i;
        z_      = z_&i  ;
        x_      = x_&i  ;
        ca      = sum(m_com#(y_=1));
        co      = sum(m_com#(y_=0));
        m_comsd = ca*(y_=1)+co*(y_=0);     * constant in (s,d);
        m_incsd = nm_&i;                   * constant in (s,d);
        *rint y_ m_com m_comsd;

      *Computation of weight vector w_i according to complete obs;
        *cases;
        w1  =m_com#(y_=1)/sum(m_com#(y_=1));
        *controls;
        w0  =m_com#(y_=0)/sum(m_com#(y_=0));
        w_i=w1+w0;

      *Save complete and completed observations ;
        y      =y//y_;
        x      =x//x_;
        z      =z//z_;
        m      =m//(m_com+m_incsd#w_i);
        mcom   =mcom//m_com;
        mcomsd =mcomsd//m_comsd;
        mincsd =mincsd//m_incsd;

      %end;    *end of i-loop over z-strata;

      * Design-matrix X for logistic model;
      X=repeat(1,nrow(y),1)||x;

    finish weight;


    start wpvar;
    *
    * Variance for weighted logistic regression
    *   according to Reilly/Pepe (see SD)
    ;

      p=1/(1+exp(-x*b));
      f=p#(1-p);
      w =   m/f;
      xx = f # x;
      %* xpxi should equal parameter covariance matrix of proc logistic;
      *  xpxi = inv(xx`*(w#xx));

      q = (mcomsd+mincsd)/mcomsd;
      h = inv(x`*(mcom#q#f#x));
      a1= x`*(mcom#q#(q-1)#(y-p)##2#x);

      do i=1 to &n_strat;
         m1i=(y=1)#mcomsd#(z=i);
         m1 =m1i[<>];
         m0i=(y=0)#mcomsd#(z=i);
         m0 =m0i[<>];
         *mean scores us;
         u1i=(y=1)#(y-p)#mcom#x#(z=i);
         u1 =u1i[+,]/m1;
         u0i=(y=0)#(y-p)#mcom#x#(z=i);
         u0 =u0i[+,]/m0       ;
         us =us//u1//u0;

         q1i=(y=1)#q#(z=i);
         q1 =q1i[<>];
         q0i=(y=0)#q#(z=i);
         q0 =q0i[<>];
         qs =qs//q1//q0;
         ms =ms//m1//m0;
      end;
      a2=us`*(qs#(qs-1)#ms#us);

      * v - Variance of mean-scor estimate;
      COV=h+h*(a1-a2)*h;

      %* Modification of intercept due to retrospective phase one sample;
      if &prosps1=0 then do;
         b[1]=b[1] - log(n1/n0);
         COV[1,1]=COV[1,1] - (1/n0+1/n1);
      end;

      stderr = sqrt(vecdiag(COV));
    finish wpvar;

    start lauf;
      run input;
      run weight;
      run wpvar;
    finish lauf;

    run lauf;

    * Output to datasets;
    create wl_temp1 var{parm b stderr};
      append;
    close wl_temp1;
    create wl_temp2 from COV[colname=parm];
      append from COV;
    close wl_temp2;

    * Combine output datasets;
    %LET FLAG=1;
        %LET ELAPS=%sysfunc(putn(%sysevalf(%sysfunc(TIME())-&DATETIME_START.),E8601TM12.2)) (hh:mm:ss.ff) ;
    data  &DAT_AUS(label="Prgm: &PRG_NAME, &sysdate"
                             drop =_day _mon _year _hour _min _sec);
    merge wl_temp1(rename=(parm=_parms b=_estim stderr=_stderr))
          wl_temp2;
      if symget("FLAG") = 1 then do;
        call symput("FLAG","0");
        _program = "&PRG_NAME";
        _source  =  "&path_ph1 (Phase 1), &path_ph2 (Phase 2)";
        _regress = "&REGRESS";
        _day     = compress(day(   date()));
        _mon     = compress(month( date()));
        _year    = compress(year(  date()));
        _hour    = compress(hour(  time()));
        _min     = compress(minute(time()));
        _sec     = compress(second(time()));
        if _day < 10 then _day =compress("0"||_day );
        if _mon < 10 then _mon =compress("0"||_mon );
        if _hour< 10 then _hour=compress("0"||_hour);
        if _min < 10 then _min =compress("0"||_min );
        if _sec < 10 then _sec =compress("0"||_sec );
        _time    = compress(_year || "." || _mon || "." || _day) || " " ||
                   compress(_hour || ":" || _min || ":" || _sec);
        _method  = "wl";
        _elaps   = "&ELAPS";
          end;
      label
        _parms   = 'name of parameter'
        _estim   = 'estimate'
        _stderr  = 'standard error'
        _source  = 'source file used by _program'
        _regress = 'regressors used by _program'
        _program = 'program that created this file'
        _time    = 'time of creation of this file'
        _method  = 'method used by _program'
        _elaps   = 'time used by _program'
      ;
    run;
    proc datasets library=work nolist;
         delete wl_temp1 wl_temp2;
    quit;

 %MEND WLR;

 %WLR_LAUF;
%MEND WLRMACRO;
