/*******************************************************************************
*                                                                              *
** PROGRAM NAME: twophase.sas                                                  *
** AUTHOR: D. Enders                                                           *
** VERSION/DATE: V 1.0 / 04.01.2012                                            *
**               V 1.1 / 03.08.2012                                            *
*                                                                              *
** PURPOSE: Macro: Analysis of two-stage studies                               *
*                                                                              *
** PROGRAM SPECIFICATION: calculates parameter estimates and model covariance  *
*                         in prospective and retrospective two-phase studies   *
*                         Comparison of results                                *
*                                                                              *
** CALLED MACROS: prep.sas                                                     *
*                 ZLRMACRO.sas                                                 *
*                 BC_MACRO.sas                                                 *
*                 EM5MACRO.sas                                                 *
*                 SW_MACRO.sas                                                 *
*                 WLRMACRO.sas                                                 *
*                 S2_MACRO.sas                                                 *
*                 TS_OUTPUT.sas                                                 *
*                 TS_COMPA.sas                                                 *
*                                                                              *
** MODIFICATION NOTES:                                                         *
*                V1.0 - V2.0: NA                                               *
** VALIDATION PROCESS:                                                         *
*                Second program review by ?                                    *
*                                                                              *
** REFERENCE FOR DOCUMENTATION: G:\DFG_SurveyMeth\DoubleSampling               *
                            \sas-twophase-package_DE\twophase_docu.pdf         *
********************************************************************************/


 %macro twophase(folder=,
                 path_ph1    =,
                 path_ph2    =,
                 methods     =pl_bc,
                 suffix      =,
                 compare     =1,
                 outproc     =0,
                 outcorr     =0,
                 outest      =0,
                 caco        =,
                 svar        =,
                 counts_ph1  =,
                 case        =1,
                 control     =0,
                 weights_ph2 =,
                 regr        =,
                 maxit       =1000,
                 epsil       =1e-10,
                 s1pr        =0) ;

/*******************************************************************************
*   MACROPARAMETERS:                                                           *
*       folder      - Path containing Macros called by %twophase               *
*       path_ph1    - Path of Phase One Data                                   *
*       path_ph2    - Path of Phase Two Data                                   *
*       methods     - names of the estimation methods (seperated by blanks)    *
*                     available methods:                                       *
*                     pl_bc  (Pseudo  Likelihood by Breslow-Cain)              *
*                     pl_sch (Pseudo  Likelihood by Schill)                    *
*                     wl     (Weighted Likelihood)                             *
*                     ml_sw  (ML via profile likelihood Scott/Wild,  WS)       *
*                     ml_em  (Maximum Likelihood via EM-Algorithmus, KD,WS)    *
*                     s2     (Analysis with Phase Two Data Only)               *
*       suffix      - suffix for Output-Data                                              *
*       compare     - 1: SAS-Output of all Parameter Estimates and Standard Errors        *
*       outproc     - 1: SAS-Output of Single Procedures                                  *
*       outcorr     - 1: SAS-Output of Correlation Matrices (outproc=1 necessary)         *
*       outest      - 1: Phase One Estimates are printed (compare=1 necessary)            *
*       caco        - Name of Case-Control Variable                                       *
*       svar        - Name of Stratum Variable                                            *
*       counts_ph1  - Name of Variable containing observation counts at Phase One         *
*       case        - Value of caco (Cases)                                               *
*       control     - Value of caco (Controls)                                            *
*       weights_ph2 - Name of weight Variable at Phase Two                                *
*       regr        - Names of the Regression Variables (seperated by blanks)             *
*       maxit       - Maximum Number of iterations, whenever iterations occur             *
*       epsil       - Accuracy of Calculations                                            *
*       s1pr        - 1: prospective sampling at Phase One                                *
*                     0: retrospective sampling at Phase One                              *
*******************************************************************************************/

%let meth_count=%sysfunc(countw(&methods));
  %DO  count=1 %TO &meth_count;
        %if %scan(&methods,&count)= ml_em  %then %let meth_macr_&count=em5macro;
        %if %scan(&methods,&count)= ml_sw  %then %let meth_macr_&count=sw_macro;
        %if %scan(&methods,&count)= pl_sch %then %let meth_macr_&count=zlrmacro;
        %if %scan(&methods,&count)= pl_bc  %then %let meth_macr_&count=bc_macro;
        %if %scan(&methods,&count)= wl     %then %let meth_macr_&count=wlrmacro;
        %if %scan(&methods,&count)= s2     %then %let meth_macr_&count=s2_macro;
  %end;;

  %PUT *******************************************************************;
  %PUT *                                                                 *;
  %PUT * Start Preparation                                               *;
  %PUT *                                                                 *;
  %PUT *******************************************************************;
  %include "&folder.\prep.sas";
  %prep(    DAT_EIN_ph1 =&path_ph1,
            DAT_EIN_ph2 =&path_ph2,
            DAT_AUS     =prep,
            strat       =&svar,
            count       =&counts_ph1,
            caco        =&caco,
            case        =&case,
            control     =&control,
            covar       =&regr,
            weight_var  =&weights_ph2);


   %DO  count=1 %TO &meth_count;
        %LET NAME=&&meth_macr_&count;
        %LET NAME2=%SCAN(&methods,&count);
          %PUT *******************************************************************;
          %PUT *                                                                 *;
          %PUT * Start &NAME;
          %PUT *                                                                 *;
          %PUT *******************************************************************;
          %include "&folder.\&name..sas";

          %&NAME(DAT_EIN   = prep,
                 DAT_AUS   = &name2._est&suffix.,
                 MAXITER   = &MAXIT,
                 EPSILON   = &EPSIL,
                 %IF &name ne %str(s2_macro) %THEN PROSPS1 =&S1PR,;
                 REGRESS   = &REGR);

          %IF &outproc ne 0 %THEN %DO;
            %INC "&folder.\ts_output.sas";
            %TS_OUTPUT(DAT_EIN =&name2._est&suffix.,
                      DOKU_COR=&outcorr);
          %END;
        %END;

      %IF &compare ne 0 %THEN %DO;
        %PUT *******************************************************************;
        %PUT *                                                                 *;
        %PUT * Comparison results of twophase methods                          *;
        %PUT *                                                                 *;
        %PUT *******************************************************************;
        %include "&folder.\ts_compa.sas";
        %TS_COMPA(%DO  count=1 %TO &meth_count;
                     %IF %SCAN(&methods,&count)=pl_sch  %THEN DAT_pl_sch  = pl_sch_est&suffix.,;
                     %IF %SCAN(&methods,&count)=pl_bc   %THEN DAT_pl_bc   = pl_bc_est&suffix.,;
                     %IF %SCAN(&methods,&count)=wl      %THEN DAT_wl      = wl_est&suffix.,;
                     %IF %SCAN(&methods,&count)=ml_sw   %THEN DAT_ml_sw   = ml_sw_est&suffix.,;
                     %IF %SCAN(&methods,&count)=ml_em   %THEN DAT_ml_em   = ml_em_est&suffix.,;
                     %IF %SCAN(&methods,&count)=s2      %THEN DAT_s2      = s2_est&suffix.,;
                  %end;
                  HILF_AUS=&outest);
      %END;

 %MEND twophase;
