
/********************************************************************
*																	*
** PROGRAMM NAME: twophase_bw.sas									*
** PROJECT		: twostage											*
** AUTHOR		: D. Ripke											*
** VERSION/DATE	: V 1.0 / 05.11.2015								*
*																	*
** PURPOSE		: Macro: backwards elimination variable selection	*
*				  for the sas-twophase-package						*
*																	*
** MACROS		: twophase.sas										*
*																	*
** VALIDATION	: Second program review by D. Enders				*
*																	*
********************************************************************/

/********************************************************************
* MACROPARAMETERS													*
* see macro twophase.sas for most parameters						*
*																	*
* bw_select - indicator whether a Backwards Elimination variable	*
*			  selection is conducted (1: yes / 0: no), default 1	*
* bw_sl		- significance level for removing effects, default 0.05	*
* bw_omit	- variables that are to be omitted from the selection	*
*			  process (separated by blanks)							*
* bw_out	- indicates whether eliminated variables are directed	*
*			  to SAS-output (1: yes / 0: no), default 1				*
*																	*
********************************************************************/

%macro twophase_bw (folder=, path_ph1=, path_ph2=, methods=pl_bc, suffix=, compare=1, outproc=0, outcorr=0, outest=0, caco=,
	svar=, counts_ph1=, case=1, control=0, weights_ph2=, regr=, maxit=1000, epsil=1e-10, s1pr=0, bw_select=1, bw_sl=0.05, bw_omit=, bw_out=1);

	%local s;									/* number of strata */
	%local regr_vars;							/* list of all variables left in at least one method */
	%let bw_omit = _ALPHA %upcase(&bw_omit);	/* variables to be omitted automatically, including intercept */
	%let c = %sysfunc(countw(&methods));		/* number of methods */

	/****************/
	/* Input Checks */
	/****************/

	/* check range of significance level &bw_sl */
	%if &bw_sl>1 | &bw_sl<0 %then %do;
		%put ERROR: Significance level BW_SL is out of bounds. The macro TWOPHASE_BW will stop executing.;
		%goto finnish_1;
	%end;

	/* check whether &bw_omit is a subset of &regr */
	%if %sysfunc(countw(&bw_omit))>1 %then %do;
		%do i=2 %to %sysfunc(countw(&bw_omit));
			%if %sysfunc(findw(%upcase(&regr), %scan(&bw_omit, &i), 'E R')) = 0 %then %do;
				%put ERROR: At least one variable in BW_OMIT is not part of REGR. The macro TWOPHASE_BW will stop executing.;
				%goto finnish_1;
			%end;
		%end;
	%end;

	/****************/
	/* No Selection */
	/****************/

	/* when no selection is requested, run analysis and jump to exit */
	%if &bw_select=0 | &bw_sl=0 %then %do;
		%twophase(folder=&folder, path_ph1=&path_ph1, path_ph2=&path_ph2, methods=&methods, suffix=&suffix, compare=&compare, outproc=&outproc, outcorr=&outcorr, outest=&outest,
			caco=&caco, svar=&svar, counts_ph1=&counts_ph1, case=&case, control=&control, weights_ph2=&weights_ph2, regr=&regr, maxit=&maxit, epsil=&epsil, s1pr=&s1pr);
		%goto finnish_1;
	%end;

   	/******************************************/
	/* Backwards-Elimination for every Method */
	/******************************************/

	/* determine length of the longest variable */
	%let max_ges = 1;
	%do i=1 %to %sysfunc(countw(&regr));
		%let var_len = %length(%scan(&regr, &i));
		%if &var_len>&max_ges %then %let max_ges=&var_len;
	%end;


%macro bw_method (curr_method=, omit=); /* BE for a single method */
/*  curr_method: currently used estimation method
	omit: variables omitted from selection */

	/****************/
	/* Preparations */
	/****************/

	/* add variables that are to be automatically omitted to &omit */
	%if &curr_method=pl_bc | &curr_method=pl_sch %then %do;
		data _null_;
			set ph1 nobs=nobs;
			call symputx('s', nobs/2);
		run;
		%do p=1 %to &s;
			%let omit = &omit _S&p;
		%end;
	%end;
	
	%let variables = %upcase(&regr);		/* variables used in the current model */
	%let syscc = 0;							/* indicator for errors */
	%let p_max = 1;							/* greatest p-value within the analysis */
	%local var_max;							/* variable with the greatest p-value */

	/* prepare dataset for output of eliminated variables */
	proc sql;
		create table elim_&t (variable&t char &max_ges, p_value&t char);
		%if &bw_out=1 & (&outproc=1 & &compare=1 & &c>1 | &outproc=0 & (&compare = 1 | &compare=0 & &c>1)) %then %do;
			insert into elim_&t set variable&t='Variable', p_value&t='P-value';
			insert into elim_&t set variable&t=' ', p_value&t=' ';
		%end;
	quit;

   	/*************************/
	/* Backwards-Elimination */
	/*************************/


%macro bw_step (curr_vars=); /* individual step within a Backwards Elimination */
/* curr_vars: variables used in the current model */

	/* run analysis with current variable list */
	%twophase(folder=&folder, path_ph1=&path_ph1, path_ph2=&path_ph2, methods=&curr_method, suffix=&suffix, compare=0, outproc=0, outcorr=0, outest=0,
		caco=&caco, svar=&svar, counts_ph1=&counts_ph1, case=&case, control=&control, weights_ph2=&weights_ph2, regr=&curr_vars, maxit=&maxit, epsil=&epsil, s1pr=&s1pr);

	/* quit loop if an error occurs in %twophase */
	%if &syscc>0 %then %do;
		%put WARNING: Error or warning in macro TWOPHASE. The macro TWOPHASE_BW will stop executing. Please check submitted parameters.;
		%goto finnish_3;
	%end;

	/* _estim is asymptotically normally distributed with true parameter theta as expected value */
	/* Wald test with null hypothesis theta=0 -> squared test statistic has chi-sqared distribution with 1 df */

	/* calculate test statistic and p-values */
	data calc;
		set &curr_method._est&suffix;
		t = (_estim/_stderr)**2;
		p = 1 - cdf('chisquare', t, 1);
		keep _parms t p;
	run;

	/* remove observations for the variables of &bw_omit */
	%do i=1 %to %sysfunc(countw(&omit));
		data calc;
			set calc;
			if _parms ^= "%scan(&omit, &i)";
		run;
	%end;

	/* determine maximum p-value and its variable (last row in resulting dataset) */
	data calc;
		set calc;
		retain p_max 0;
		p_max = max(p, p_max);
		if p=p_max;
		p_max = round(p_max, 0.00001);
		p_max_char = put(p_max, 7.5);
	run;

	data _null_;
		set calc;
		call symputx('p_max', p_max_char);
		call symput('var_max', _parms);
	run;

	/* if &p_max is not less than the significance level: */
	%if &p_max >= &bw_sl %then %do;
		/* delete &var_max from current variable list */
		%let variables = %sysfunc(tranwrd(&curr_vars, &var_max, ));
		/* add &var_max to the output dataset */
		proc sql;
			insert into elim_&t set variable&t="&var_max", p_value&t="&p_max";
		quit;
		/* set indicator for elimination of variables to 1 */
		%let elim_&t = 1;
	%end;

	%finnish_3:

%mend bw_step;


	/* execute BE loop as long as the maximum p-value is not less than the significance level */
	%do %while (&p_max >= &bw_sl);
		%bw_step(curr_vars=&variables);
		/* abort macro if an error occurs in %twophase */
		%if &syscc>0 %then %goto finnish_2;
	%end;

	/***************************************************/
	/* End Analysis for the Method and optional Output */
	/***************************************************/

	%let variables = %sysfunc(compbl(&variables));

	%twophase(folder=&folder, path_ph1=&path_ph1, path_ph2=&path_ph2, methods=&curr_method, suffix=&suffix, compare=0, outproc=&outproc, outcorr=&outcorr, outest=&outest,
		caco=&caco, svar=&svar, counts_ph1=&counts_ph1, case=&case, control=&control, weights_ph2=&weights_ph2, regr=&variables, maxit=&maxit, epsil=&epsil, s1pr=&s1pr);

	%do i=1 %to %sysfunc(countw(&variables));
		%let rv = %scan(&variables, &i);
		%if %sysfunc(findw(&regr_vars, &rv))=0 %then %let regr_vars = &regr_vars &rv;
	%end;

	data est_&t;
		set &curr_method._est&suffix;
		keep _parms _estim _stderr;
		rename _estim=_estim&t _stderr=_stderr&t;
	run;

	proc sort data=est_&t;
		by _parms;
	run;

	/* output of the eliminated variables for individual output per method */
	%if &bw_out=1 & (&outproc=0 & &compare=0 & &c=1 | &outproc=1 & (&compare=0 | &compare=1 & &c=1)) %then %do;

		options formdlim=' ';
		%if &&elim_&t=1 %then %do;
			data elim_&t;
				set elim_&t;
				label variable&t=Variable p_value&t=P-value;
			run;
			title "Method &curr_method.: Eliminated Variables in Order of Exclusion";
			proc print data=elim_&t label obs='Step';
			run;
		%end;
		%else %if &&elim_&t=0 %then %do;
			title ' ';
			options nodate nonumber;
			data _null_;
				file print;
				put "No variables were eliminated in method &curr_method..";
			run;
			options date number;
		%end;
		title 'The SAS System';
		options formdlim='';

	%end;

	/* delete auxiliary datasets */
	proc datasets nolist;
		delete calc;
	run;

	%finnish_2:

%mend bw_method;


	%do t=1 %to &c;
		%let elim_&t = 0;	/* indicator for eliminated variables in method t */
		%bw_method(curr_method=%scan(&methods, &t), omit=&bw_omit);
		/* abort macro if an error occurs in macro %twophase */
		%if &syscc>0 %then %goto finnish_1;
	%end;

   	/**********/
	/* Output */
	/**********/

	/* final output of analysis for &compare=1 */
	%if &compare=1 %then %do;

		data est_all;
			merge est_1-est_&c;
			by _parms;
		run;

		/* delete auxiliary variables if necessary */
		%if &outest=0 & (%sysfunc(findw(&methods, pl_sch))>0 | %sysfunc(findw(&methods, pl_bc))>0) %then %do p=1 %to &s;
			data est_all;
				set est_all;
				if _parms ^= "_S&p";
			run;
		%end;
	
		/* determine length of longest variable still remaining */
		%let max_len = %length(_alpha);
		%do i=1 %to %sysfunc(countw(&regr_vars));
			%let var_len = %length(%scan(&regr_vars, &i));
			%if &var_len>&max_len %then %let max_len=&var_len;
		%end;
	
		/* prepare method headlines */
		%let headline = %str( );
		%do l=1 %to &max_len;
			%let headline = &headline%str( );
		%end;
		%let headline1 = %str("&headline ");
		%let headline2 = %str("&headline ");

		%do t=1 %to &c;
			%if %scan(&methods, &t)=pl_sch %then %let met_name&t = %str("PL (Schill)          ");
			%if %scan(&methods, &t)=pl_bc  %then %let met_name&t = %str("PL (Breslow-Cain)    ");
			%if %scan(&methods, &t)=wl     %then %let met_name&t = %str("Weighted Regression  ");
			%if %scan(&methods, &t)=ml_em  %then %let met_name&t = %str("ML (EM-Algorithm)    ");
			%if %scan(&methods, &t)=ml_sw  %then %let met_name&t = %str("ML (Profile Lik.)    ");
			%if %scan(&methods, &t)=s2     %then %let met_name&t = %str("Sample2-Analysis     ");
		%end;
	
		%let ls_left = %eval(%sysfunc(getoption(linesize))-(&max_len+1));
		
		proc iml;
			start input;
				use est_all;
					read all var {_parms} into parms;
					%do t=1 %to &c;
						read all var {_estim&t} into estim&t;
						read all var {_stderr&t} into stderr&t;
					%end;
				close est_all;
			finish input;
			start output;
				%do t=1 %to &c;
					%if &ls_left<22 %then %let break=1; %else %let break=0;
					%if &break=0 %then %do;
						%let headline1 = &headline1 &&met_name&t;
						result1 = result1 || estim&t || stderr&t;
						cols1 = cols1 || {'estim' 'stderr'};
					%end;
					%else %if &break=1 %then %do;
						%let headline2 = &headline2 &&met_name&t;
						result2 = result2 || estim&t || stderr&t;
						cols2 = cols2 || {'estim' 'stderr'};	
					%end;
					%let ls_left = %eval(&ls_left-22);
				%end;
				reset noname;
				print /&headline1, result1[rowname=parms colname=cols1 format=10.5];
				%if %length(&headline2)>%eval(&max_len+5) %then %do;
					print /&headline2, result2[rowname=parms colname=cols2 format=10.5];
				%end;
				reset name;
			finish output;
			run input;
			run output;
		quit;

	%end;

	/* final output of eliminated variables for collective output of all methods */
	%if &bw_out=1 & (&outproc=1 & &compare=1 & &c>1 | &outproc=0 & (&compare = 1 | &compare=0 & &c>1)) %then %do;

		/* output when no variables were eliminated in any method */
		%let elim_all = 0;
		%do t=1 %to &c;
			%let elim_all = %eval(&elim_all + &&elim_&t);
		%end;
		%if &elim_all=0 %then %do;
			options formdlim=' ' nodate nonumber;
			title ' ';
			data _null_;
				file print;
				put "No variables were eliminated in any method.";
			run;
			title 'The SAS System';
			options formdlim='' date number;
			%goto end_output;
		%end;

		/* output when variables were eliminated in at least one method */

		%local methods_elim;	/* methods with eliminated variables */
		%local methods_noelim;	/* methods with no eliminated variables */
		%let i=1;

		/* rearrange datasets with eliminated variables */
		%do t=1 %to &c;
			%if &&elim_&t = 0 %then %let methods_noelim = &methods_noelim %scan(&methods, &t);
			%else %if &&elim_&t = 1 %then %do;
				%let methods_elim = &methods_elim %scan(&methods, &t);
				data elim_new_&i;
					set elim_&t;
					rename variable&t = variable&i p_value&t = p_value&i;
				run;
				%let i = %eval(&i+1);
			%end;
		%end;

		%let d = %sysfunc(countw(&methods_elim));

		/* prepare method headlines */
		%do b=1 %to &d;
			%if %scan(&methods_elim, &b)=pl_sch %then %let met_new_name_&b = %str("PL (Schill)          ");
			%if %scan(&methods_elim, &b)=pl_bc  %then %let met_new_name_&b = %str("PL (Breslow-Cain)    ");
			%if %scan(&methods_elim, &b)=wl     %then %let met_new_name_&b = %str("Weighted Regression  ");
			%if %scan(&methods_elim, &b)=ml_em  %then %let met_new_name_&b = %str("ML (EM-Algorithm)    ");
			%if %scan(&methods_elim, &b)=ml_sw  %then %let met_new_name_&b = %str("ML (Profile Lik.)    ");
			%if %scan(&methods_elim, &b)=s2     %then %let met_new_name_&b = %str("Sample2-Analysis     ");
		%end;

		/* merge datasets with eliminated variables */
		data elim_new_all;
			merge elim_new_1-elim_new_&d;
			%do b=1 %to &d;
				var&b = cat(variable&b, '    ', p_value&b);
				label var&b = &&met_new_name_&b;
			%end;
			keep var1-var&d;
		run;

		data elim_new_all;
			retain Step;
			set elim_new_all;
			Step = put(_n_-2, 8.);
			if Step<1 then Step=' ';
		run;

		title 'Eliminated Variables in Order of Exclusion';
		proc print data=elim_new_all rows=page label noobs;
		run;

		%if &elim_all^=&c %then %do;
			options formdlim=' ' nodate nonumber;
			title ' ';
			data _null_;
				file print;
				put "No variables were eliminated in method(s) &methods_noelim..";
			run;
			options formdlim='' date number;
		%end;
		title 'The SAS System';

		/* delete auxiliary datasets */
		proc datasets nolist;
			delete elim_new_1-elim_new_&d elim_new_all;
		run;

		%end_output:

	%end;

	/* delete auxiliary datasets */
	proc datasets nolist;
		delete est_1-est_&c est_all elim_1-elim_&c;
	quit;

	%finnish_1:

%mend twophase_bw;

