/******************************************************************************************
*                                                                                         *
**PROJECT:   2-Phase Transfer															  *
*                                                                                         *
**PROGRAM NAME: extendedPL.sas			                                                  *
*                                                                                         *
**AUTHOR:       D. Enders					                                     	      *
*                                                                                         *
**VERSION/DATE: V 1.0 / 06.03.2017							 							  *
*                                                                                         *
**PURPOSE:  Implementation of extended Pseudo likelihood approach for 2phase case-control *
*			studies																		  *
*                                                                                         *
**VALIDATION PROCESS: Single Programmer evolvement		                         		  *
*                                                                                         *
** REFERENCE FOR DOCUMENTATION: TP_DOCU_extendedPL.pdf									  *
*								Scott and Wild (2011). Fitting regression models with 	  *
*								response-biased samples. The Canadian Journal of 		  *
*								Statistics 39 (3): 519-536  							  *
*               												 						  *
*****************************************************************************************/
%macro extPL(	path_in=,
				  sel_var=, 
				  out_var=, 
				  caco=,
				  ph2_ind=, 
				  path_out=, 
				  diag=);

/**********************************************************************************************************
* Explanation of parameters:											  	   						      *
*																			   						      *
* path_in	= Path to input data														   				  *
* sel_var	= List of covariates of selection model 							   						  *
* out_var	= List of covariates of outcome model								   						  *
* caco		= Indicator of cases (1=case, 0=control)								   					  *
* ph2_ind	= Indicator of selection in phase 2 (1=selected, 0=not selected)	   						  *
* path_out	= Name of output dataset												   					  *
* diag		= Indicator, if details of procedures/extrapolations should be printed (0=no (default), 1=yes)* 
**********************************************************************************************************/

	%*extract phase 2;
	data phase2;
		set &path_in(where=(&ph2_ind=1));
		call symput('N_ph2',_n_);
	run;

    proc sort data=phase2 out=phase2;
     by &sel_var;
    run;

	%*Calculate selection probabilities for phase 2, seperately for cases and controls;
	proc sort data=&path_in out=phase1;
		by &caco;
	run;

	title1"Regression for Participation model";
	proc logistic data=phase1 %if &diag=0 %then %do; noprint %end; covout descending outest=est_alpha;
		by &caco;
		model &ph2_ind= &sel_var;
		output out=ana_weight prob=pi;
	run;
	title1;

	%*Extract Covariance matrices for cases and controls;
	data cov_alpha0;
		set est_alpha;
		where &caco=0 and _type_="COV";
		keep intercept &sel_var;
	run;

	data cov_alpha1;
		set est_alpha;
		where &caco=1 and _type_="COV";
		keep intercept &sel_var;
	run;

	%*Store estimates of the selection model parameters for cases and controls;
	data alpha0;
		set est_alpha;
		if &caco=0 and _type_="PARMS";
		keep intercept &sel_var;
	run;

	data alpha1;
		set est_alpha;
		if &caco=1 and _type_="PARMS";
		keep intercept &sel_var;
	run;


   %LET NP=0;
   %DO %WHILE(%SCAN(&sel_var,&NP+1)^=);
	   %LET NP=%EVAL(&NP+1);
	   %LET Z&NP=%SCAN(&sel_var,&NP);
   %END;

   data _null_;
	 set alpha0;
     call symput('intercept0',intercept);
     %do i=1 %to &np;
        call symput("est0_&i",&&Z&i);
     %end;
	run;

   data _null_;
	 set alpha1;
     call symput('intercept1',intercept);
     %do i=1 %to &np;
        call symput("est1_&i",&&Z&i);
     %end;
   run;

   %*Obtain different covariate patterns in phase 2, seperately for cases and controls;
   %*for each covariate pattern in &sel_var;
   data w;
   	  set ana_weight;
	  if &ph2_ind=1;
	run;

   proc sort data=w nodupkey;
     by &caco &sel_var;
   run;

   data w0 (rename=(pi=pi0)) w1 (rename=(pi=pi1));
      set w;
	  by &caco &sel_var;
      keep &sel_var pi;
      if &caco=0 then output w0;
      else         output w1;
   run;

   %*Calculate offset omega for each covariate pattern, obtain number of extrapolations for cases and controls with missing cov-pattern;
   data w2;
	  merge w0 w1;
      by &sel_var;
      rep0=0;
	  rep1=0;
      label rep0 = 'Indicator for missing pi0 (1: yes)'
            rep1 = 'Indicator for missing pi1 (1: yes)';
      %* If pi1=. or pi0=., replace by appropriate extrapolation;
      if pi0=. then do;
         rep0=1;
         eta0= &intercept0 %do i=1 %to &np; + &&est0_&i.*&&Z&i %end;;
         pi0 =1/(1+exp(-eta0));
      end;
      if pi1=. then do;
         rep1=1;
         eta1= &intercept1 %do i=1 %to &np; + &&est1_&i.*&&Z&i %end;;
         pi1 =1/(1+exp(-eta1));
      end;
      omega=log(pi1/pi0);
   run;

   %if &diag.^=0 %then %do;
	   %*Display number of missing cov-patterns;
	   title1'Offstes per covariate-pattern';
	   proc print data=w2;
	     var &sel_var  pi0 rep0 pi1 rep1 omega;
	     sum rep0 rep1;
	   run;

	   title1'Replacements per outcome-status';
	   proc freq data=w2;
	     tables rep0*rep1/nopercent norow nocol;
	   run;
	   title1;
  %end;
   %*attach omega to phase 2 data;
   data phase2;
	  merge phase2 w2;
      by &sel_var;
   run;

   %*Conduct extended pseudo-likelihood method;
   title1"Regression for Outcome model";
   title2"Selection to phase 2 depends on &sel_var";
   proc logistic data=phase2 %if &diag=0 %then %do; noprint %end; descending covout outest=est_beta outmodel=cov;
     model &caco=&out_var /offset=omega;
   run;
   title1;
   title2;

   %*Save parameter estimates and covariances from logistic regression;
	data cov_beta;
		set est_beta;
		where _type_="COV" and _name_^="omega";
		keep _name_ intercept &out_var;
	run;

	data beta;
		set est_beta;
		if _type_="PARMS";
		keep intercept &out_var;
	run;

   %*Calculate covariance matrix which includes estimation of participation probabilities;
   proc iml;
   start input;
   	  %*read Design matrix for outcome and selection model, and weights;
 	  use phase2;
          read all var {&out_var} into x_beta;
          read all var {&sel_var} into x_alpha;
          read all var {omega}    into omega;
      close phase2;

   	  %*read parameter estimates and covariances from logistic regression for selection;
	  use alpha0;
		  read all var{intercept &sel_var} into alpha0;
	  close alpha_0;

	  use alpha1;
		  read all var{intercept &sel_var} into alpha1;
	  close alpha_1;

	  use cov_alpha0;
	  		read all var {intercept &sel_var} into cov_alpha0;
	  close cov_alpha0;

	  use cov_alpha1;
	  		read all var {intercept &sel_var} into cov_alpha1;
	  close cov_alpha1;

	  %*read parameter estimates and covariance matrix form logistic regression for outcome;
	  use beta;
	  	  read all var {intercept &out_var} into beta;
	  close beta;

	  use cov_beta;
	  		read all var {intercept &out_var} into cov_beta;
			read all var {_name_} into names;
	  close cov_beta;	
	
 	finish input;
	run input;
	  %*calculations according to Scott and Wild (2011);
	  D_alpha=repeat(1,&N_ph2.)||x_alpha;
	  D_beta=repeat(1,&N_ph2.)||x_beta;
	  mu=exp(D_beta*t(beta)+omega)/(1+exp(D_beta*t(beta)+omega));
	  mu_alpha1=exp(D_alpha*t(alpha1))/(1+exp(D_alpha*t(alpha1)));
	  mu_alpha0=exp(D_alpha*t(alpha0))/(1+exp(D_alpha*t(alpha0)));
	  S_alpha1=D_alpha#(1-mu_alpha1);
	  S_alpha0=D_alpha#(1-mu_alpha0);
	  I01_1=t(D_beta)*diag(mu#(1-mu))*S_alpha1;
	  I01_0=t(D_beta)*diag(mu#(1-mu))*S_alpha0;
	  COV=cov_beta-cov_beta*I01_0*cov_alpha0*t(I01_0)*cov_beta-cov_beta*I01_1*cov_alpha1*t(I01_1)*cov_beta;
	  stderr = sqrt(vecdiag(COV));
	  stderr_rob = sqrt(vecdiag(cov_beta)); %*this is the robust alternative;
	  create out var{names beta stderr stderr_rob};
      append;
quit;
	%*Creation of output dataset;
	data &path_out(rename=(names=Variable));
		retain names Est Stderr Stderr_rob;
		set out;
		Est=round(beta,0.001);
		Stderr=round(stderr,0.001);
		Stderr_rob=round(stderr_rob,0.001);
		names=upcase(names);
		if names="INTERCEPT" then names="_ALPHA";
		drop beta;
	run;

	proc datasets;
		delete out 
			   cov_alpha0
			   cov_alpha1
			   alpha0
			   alpha1
			   beta
			   cov_beta
			   w
			   w0
			   w1
			   w2
			   ana_weight
			   cov
			   est_alpha
			   est_beta
			   phase1
			   phase2;
	quit;
%mend extPL;


